const constants = require('./constants');
const TrackviaAPI = require('trackvia-api');
const formatRecord = require('./services/formatRecord');
const mergeRecord = require('./services/mergeRecord');

const fs = require('fs');

exports.handler = async (event, context, callback) => {
    context.callbackWaitsForEmptyEventLoop = false;
    console.log('EVENT:', event);

    const api = new TrackviaAPI(constants.key, constants.token, constants.host, constants.accountId);

    // Fetch the record that was triggered
    let schema;
    let record;
    try {
        const response = await api.getRecord(constants.mergingViewId, event.recordId);
        schema = response.structure;
        record = response.data;
    } catch (err) {
        console.log('Could not fetch record');
        callback(err);
        return false;
    }

    // Check that the record is intending a merge
    if(record[constants.templateFieldName] !== constants.templateName) {
        callback(null, 'No record to merge');
        return true;
    }

    // Format each field based on type
    let formstackPayload = await formatRecord(constants.mergingViewId, record, schema);

    // Handle child record views
    if(constants.childViews.length) {
        await Promise.allSettled(constants.childViews.map(async childView => {
            // Fetch potentional child records
            const results = await api.getView(childView.viewId, null, record[childView.searchParamField]);
            // Ensure that the right child records are selected
            const children = results.data.filter(result => result[`${childView.linkToParentRelationshipName}(id)`] === record.id);
            formstackPayload[childView.templateVariable] = await Promise.allSettled(children.map(async child => await formatRecord(child, results.structure)));
        }));
    }

    // Send the formatted data to formstack for processing
    let formstackResponse;
    try {
        formstackResponse = await mergeRecord(constants.formstackMergeUrl, formstackPayload);
    } catch (err) {
        console.log('Could not merge record');
        callback(err);
        return false;
    }

    // Create a record to hold the generated document
    let destination;
    try {
        const response = await api.addRecord(constants.mergedViewId, {
            [constants.mergedTableTemplateFieldName]: constants.templateName,
            [constants.mergedTableRelationshipName]: record.id
        });
        destination = response.data.pop();
    } catch (err) {
        console.log('Could not create record for document');
        callback(err);
        return false;
    }

    // Attach finished document to newly created file
    try {
        // Write image to local  file for upload
        const filePath = `/tmp/${constants.mergedDocumentName(record)}`;
        fs.writeFileSync(filePath, formstackResponse, 'binary');
        await api.attachFile(constants.mergedViewId, destination.id, constants.mergedTableDocumentFieldName, filePath);
        fs.unlinkSync(filePath);
    } catch (err) {
        console.log('Could not attach generated document');
        callback(err);
        return false;
    }

    // Reset the document merge value
    try {
        await api.updateRecord(constants.mergingViewId, record.id, {
            [constants.templateFieldName]: null,
            // 'Last Weekly Report Run Date': moment().toISOString(),
        });
    } catch (err) {
        console.log('Could not reset template dropdown');
        callback(err);
        return false;
    }

    callback(null, 'success');
    return true;
};
